var Store = require('trevor-store').Store;
var bandcamp = require('bandcamp-scraper');
var diacritics = require('diacritics');
var async = require('async');


function searchArtist(term, cb) {
  bandcamp.search({query: term}, function (error, searchResults) {
    if (error) return cb(error, null);
    term = diacritics.remove(term).toLowerCase();
    var artist = null;
    var i = searchResults.length;
    while (artist == null && i--) {
      if (searchResults[i].type == 'artist') {
        var artistName = diacritics.remove(searchResults[i].name).toLowerCase();
        if (artistName == term) {
          artist = searchResults[i];
        }
      }
    }
    cb(null, artist);
  })
}

// function for async
function getAlbumProducts(albumUrl, cb) {
  bandcamp.getAlbumProducts(albumUrl, cb);
}

function getProductFormat(format) {
  if (/digital/i.exec(format)) {
    return 'Digital';
  } else if (/(cd|compact disc)/i.exec(format)) {
    return 'CD';
  } else if (/vinyl/i.exec(format)) {
    return 'Vinyl';
  } else {
    // TODO include merch? Not for the moment
    // console.log('Cannot parse format ' + format);
    // return 'Other';
  }
}

module.exports = new Store({

  storeName: 'Bandcamp',
  artist: null,

  request: function (term, resolve, reject) {
    searchArtist(term, function (error, artist) {
      if (artist) {
        bandcamp.getAlbumUrls(artist.url, function (error, albumUrls) {
          if (albumUrls) {
            async.map(albumUrls, getAlbumProducts, function(error, products){
              if (products) {
                products = products.reduce(function (flattenProducts, products) {
                  return flattenProducts.concat(products);
                }, []);
                resolve(products);
              }
            });
          }
        });
      } else {
        reject("Can't find artist!");
      }
    }.bind(this));
  },

  formatItem: function (product) {
    var format = getProductFormat(product.format);
    if (product.soldOut === false && format) {
      return {
        artist: product.artist,
        title: product.name,
        store: this.storeName,
        priceInCents: product.priceInCents || 0,
        currencyCode: product.currency,
        url: product.url,
        imageUrl: (product.imageUrls.length > 0 ? product.imageUrls[0] : null),
        isUsed: false,
        format: [ format ]
      };
    }
  }

});
